@@ -8,6 +8,7 @@ import logging
 import re
 
 import requests
+from urllib.parse import urlparse, urlunparse
 from SPARQLWrapper import SPARQLWrapper
 from boto3 import Session
 from botocore.session import Session as botocoreSession
@@ -77,6 +78,8 @@ STREAM_AT = 'AT_SEQUENCE_NUMBER'
 STREAM_AFTER = 'AFTER_SEQUENCE_NUMBER'
 STREAM_TRIM = 'TRIM_HORIZON'
 STREAM_LATEST = 'LATEST'
+STREAM_COMMIT_TIMESTAMP = 'commitTimestamp'
+STREAM_IS_LASTOP = 'isLastOp'
 STREAM_EXCEPTION_NOT_FOUND = 'StreamRecordsNotFoundException'
 STREAM_EXCEPTION_NOT_ENABLED = 'UnsupportedOperationException'
 
@@ -91,23 +94,45 @@ STREAM_ENDPOINTS = {STREAM_PG: 'gremlin', STREAM_RDF: 'sparql'}
 
 class Client(object):
     def __init__(self, host: str, port: int = DEFAULT_PORT, ssl: bool = True, region: str = DEFAULT_REGION,
-                 sparql_path: str = '/sparql', gremlin_traversal_source: str = 'g', auth=None, session: Session = None):
-        self.host = host
-        self.port = port
+                 sparql_path: str = '/sparql', gremlin_traversal_source: str = 'g', auth=None, session: Session = None,
+                 proxy_host: str = '', proxy_port: int = DEFAULT_PORT):
+        self.target_host = host
+        self.target_port = port
         self.ssl = ssl
         self.sparql_path = sparql_path
         self.gremlin_traversal_source = gremlin_traversal_source
         self.region = region
         self._auth = auth
         self._session = session
+        self.proxy_host = proxy_host
+        self.proxy_port = proxy_port
 
         self._http_protocol = 'https' if self.ssl else 'http'
         self._ws_protocol = 'wss' if self.ssl else 'ws'
 
         self._http_session = None
 
-    def get_uri_with_port(self):
-        uri = f'{self._http_protocol}://{self.host}:{self.port}'
+    @property
+    def host(self):
+        if self.proxy_host != '':
+            return self.proxy_host
+        return self.target_host
+
+    @property
+    def port(self):
+        if self.proxy_host != '':
+            return self.proxy_port
+        return self.target_port
+
+    def is_neptune_domain(self):
+        return "neptune.amazonaws.com" in self.target_host
+
+    def get_uri_with_port(self, use_websocket=False, use_proxy=False):
+        protocol = self._http_protocol
+        if use_websocket is True:
+            protocol = self._ws_protocol
+
+        uri = f'{protocol}://{self.host}:{self.port}'
         return uri
 
     def sparql_query(self, query: str, headers=None, explain: str = '', path: str = '') -> requests.Response:
@@ -139,7 +164,7 @@ class Client(object):
             sparql_path = f'/{path}'
         elif self.sparql_path != '':
             sparql_path = f'/{self.sparql_path}'
-        elif "amazonaws.com" in self.host:
+        elif self.is_neptune_domain():
             sparql_path = f'/{SPARQL_ACTION}'
         else:
             sparql_path = ''
@@ -180,19 +205,18 @@ class Client(object):
             raise ValueError('query_id must be a non-empty string')
         return self._query_status('sparql', query_id=query_id, silent=silent, cancelQuery=True)
 
-    def get_gremlin_connection(self) -> client.Client:
+    def get_gremlin_connection(self, transport_kwargs) -> client.Client:
         nest_asyncio.apply()
 
-        uri = f'{self._http_protocol}://{self.host}:{self.port}/gremlin'
-        request = self._prepare_request('GET', uri)
+        ws_url = f'{self.get_uri_with_port(use_websocket=True)}/gremlin'
+        request = self._prepare_request('GET', ws_url)
+        traversal_source = 'g' if self.is_neptune_domain() else self.gremlin_traversal_source
+        return client.Client(ws_url, traversal_source, headers=dict(request.headers), **transport_kwargs)
 
-        ws_url = f'{self._ws_protocol}://{self.host}:{self.port}/gremlin'
-
-        traversal_source = 'g' if "neptune.amazonaws.com" in self.host else self.gremlin_traversal_source
-        return client.Client(ws_url, traversal_source, headers=dict(request.headers))
-
-    def gremlin_query(self, query, bindings=None):
-        c = self.get_gremlin_connection()
+    def gremlin_query(self, query, transport_args=None, bindings=None):
+        if transport_args is None:
+            transport_args = {}
+        c = self.get_gremlin_connection(transport_args)
         try:
             result = c.submit(query, bindings)
             future_results = result.all()
@@ -213,7 +237,7 @@ class Client(object):
         if headers is None:
             headers = {}
 
-        uri = f'{self._http_protocol}://{self.host}:{self.port}/gremlin'
+        uri = f'{self.get_uri_with_port()}/gremlin'
         data = {'gremlin': query}
         req = self._prepare_request('POST', uri, data=json.dumps(data), headers=headers)
         res = self._http_session.send(req)
@@ -304,7 +328,7 @@ class Client(object):
         driver = GraphDatabase.driver(url, auth=(user, password), encrypted=self.ssl)
         return driver
 
-    def stream(self, url, **kwargs) -> requests.Response: 
+    def stream(self, url, **kwargs) -> requests.Response:
         params = {}
         for k, v in kwargs.items():
             params[k] = v
@@ -631,12 +655,14 @@ class Client(object):
 
     def _prepare_request(self, method, url, *, data=None, params=None, headers=None, service=NEPTUNE_SERVICE_NAME):
         self._ensure_http_session()
+        if self.proxy_host != '':
+            headers = {} if headers is None else headers
+            headers["Host"] = self.target_host
         request = requests.Request(method=method, url=url, data=data, params=params, headers=headers, auth=self._auth)
         if self._session is not None:
             aws_request = self._get_aws_request(method=method, url=url, data=data, params=params, headers=headers,
                                                 service=service)
             request.headers = dict(aws_request.headers)
-
         return request.prepare()
 
     def _get_aws_request(self, method, url, *, data=None, params=None, headers=None, service=NEPTUNE_SERVICE_NAME):
@@ -709,5 +735,13 @@ class ClientBuilder(object):
         self.args['session'] = session
         return ClientBuilder(self.args)
 
+    def with_proxy_host(self, host: str):
+        self.args['proxy_host'] = host
+        return ClientBuilder(self.args)
+
+    def with_proxy_port(self, proxy_port: int):
+        self.args['proxy_port'] = proxy_port
+        return ClientBuilder(self.args)
+
     def build(self) -> Client:
         return Client(**self.args)
