@@ -70,7 +70,7 @@ def check_ml_enabled():
         "GET", url=f'https://{host}:{port}/ml/modeltraining', service='neptune-db')
     if response.status_code != 200:
         print('''This Neptune cluster \033[1mis not\033[0m configured to use Neptune ML.
-Please configure the cluster according to the Amazpnm Neptune ML documentation before proceeding.''')
+Please configure the cluster according to the Amazon Neptune ML documentation before proceeding.''')
     else:
         print("This Neptune cluster is configured to use Neptune ML")
 
@@ -321,7 +321,7 @@ class MovieLensProcessor:
         if not os.path.exists(f'{HOME_DIRECTORY}/data/formatted'):
             os.makedirs(f'{HOME_DIRECTORY}/data/formatted')
         # Download the MovieLens dataset
-        url = 'http://files.grouplens.org/datasets/movielens/ml-100k.zip'
+        url = 'https://files.grouplens.org/datasets/movielens/ml-100k.zip'
         r = requests.get(url, allow_redirects=True)
         open(os.path.join(self.raw_directory, 'ml-100k.zip'), 'wb').write(r.content)
 
